
% set seed

rng(12345)

% working directory

cd '/home/osterhau/Nonparametric_Fox/Code_Matlab/'

% specify number of alternatives (j), and product characteristics (k)

j = 4;    k = 2;   E = sqrt(10000);    M = 20;       nFolds = 10;


% specify moments of normal components 

means = [-2.2 -2.2; 1.3 1.3];

sigmas = cat(3, [0.8 0.15; 0.15 0.8], [0.8 0.15; 0.15 0.8]);

probs = [0.5 0.5];


% generate gaussian mixture distribution

distr = gmdistribution(means, sigmas, probs);

% set up a grid to evaluate estimated distribution

[b1,b2] = meshgrid(linspace(-4.5, 3.5, sqrt(E)), linspace(-4.5, 3.5, sqrt(E)));

gridE = [b1(:), b2(:)];

% calculate true joint CDF

trueCDF = cdf(distr, gridE);

% specify parameter of Monte Carlo experiments

n = repelem([1000, 10000], 4)'; 

R = repmat([25, 50, 100, 250]', 2, 1);

% empty matrix to save results  

resultsFKRB = [n R zeros(8, 8)];   resultsMSE = [n R zeros(8, 9)];

resultsOneSe = resultsMSE;          resultsLL = resultsMSE;


% loop over different designs of Monte Carlo study

for r = 1:size(resultsFKRB, 1)
    
    % specify number of observation units and number of grid points
    
    n = resultsFKRB(r, 1);          R = resultsFKRB(r, 2); 
  
    % empty matrix to save results of Monte Carlo experiment r
    
    resultsFKRB_r = zeros(M, 7); resultsOneSe_r = zeros(M, 8);   resultsMSE_r = zeros(M, 8);   resultsLL_r = zeros(M, 8);
    
    
    % set up a grid for estimation 
    
     haltonGrid = haltonset(k);

     gridR = 8 * (net(haltonGrid, R) - 0.5) - 0.5;

     % calculate true and empirical density at every grid point
     
     thetaTrue = pdf(distr, gridR); 
     
     thetaTrue = thetaTrue / sum(thetaTrue);
     
     thetaTrue(thetaTrue < 1e-3) = 0;
     
     thetaNorm = sum(thetaTrue > 0);
     
     S = sum(thetaNorm > 0);  resultsFKRB(r, 3) = S; resultsENet(r, 3) = S; 
     
          
    % start replicates of Monte Carlo experiment r
    
    for m = 1:M
        
        % draw random coefficients from true distribution

        betasTrue = random(distr, n);
     
        
        % generate data
    
        [xdata, dependentData] = gendata(n, j, betasTrue);
    
        
        % calculate choice probabilities evaluated at grid points
    
        linearIndexR = xdata * gridR';
    
        expLinearIndexR = exp(linearIndexR);
    
        expLinearIndexSumR = 1 + repelem(eye(n, n), 1, j) * expLinearIndexR;
    
        choiceProbsR = expLinearIndexR ./ repelem(expLinearIndexSumR, j, 1);
   

        % estimate weights with FKRB estimator and calculate estimated joint CDF
        
        [thetaFKRB, conv] = estimator(choiceProbsR, dependentData, 0, 1e-7);
        
        cdfFKRB = estcdf(thetaFKRB, gridR, gridE);
        
        % calculate RMISE, IAE and L1 and L2 norm
        
        resultsFKRB_r(m, 1:2) = [mean((cdfFKRB - trueCDF).^2) mean(abs(cdfFKRB - trueCDF))];
        
        resultsFKRB_r(m, 3:4) = [sqrt(mean((thetaFKRB - thetaTrue).^2)) mean(abs(thetaFKRB - thetaTrue))];
   
        % number of positive, true positive and sign consistent weights 
        
        thetaFKRB = thetaFKRB .* (thetaFKRB > 1e-3);
        
        resultsFKRB_r(m, 5:7) = [sum(thetaFKRB > 0) (sum((thetaFKRB > 0) .* (thetaTrue > 0)) / S)*100 (sum(sign(thetaFKRB) == sign(thetaTrue)) / R)*100];
        
        
        
        % conduct k-fod cross validation to find optimal tuning parameter
        
        mus = [0 GlmnetLambdas(choiceProbsR, dependentData)]';
        
        [muMSE, muOneSe, muLL] = crossValidation(choiceProbsR, dependentData, mus, nFolds, j);
        
                
        % estimate with generalized estimator using different tuning parameter 
        
        % OneSe
        
        [thetaOneSe, ~] = estimator(choiceProbsR, dependentData, muOneSe, 1e-7);

        cdfOneSe = estcdf(thetaOneSe, gridR, gridE);
        
        % calculate RMISE IAE, L1 and L2 norm
        
        resultsOneSe_r(m, 1:2) = [mean((cdfOneSe - trueCDF).^2) mean(abs(cdfOneSe - trueCDF))];
        
        resultsOneSe_r(m, 3:4) = [sqrt(mean((thetaOneSe - thetaTrue).^2)) mean(abs(thetaOneSe - thetaTrue))];
    
        % number of positive, true positive and sign consistent weights 
        
        thetaOneSe = thetaOneSe .* (thetaOneSe > 1e-3);
        
        resultsOneSe_r(m, 5:7) = [sum(thetaOneSe > 0) (sum((thetaOneSe > 0) .* (thetaTrue > 0)) / S)*100 (sum(sign(thetaOneSe) == sign(thetaTrue)) / R)*100];
        
        % save tuning parameter 
        
        resultsOneSe_r(m, 8) = muOneSe;
        
        % MSE
        
        [thetaMSE, ~] = estimator(choiceProbsR, dependentData, muMSE, 1e-7);

        cdfMSE = estcdf(thetaMSE, gridR, gridE);
        
        % calculate RMISE, IAE, L1 and L2 norm
        
        resultsMSE_r(m, 1:2) = [mean((cdfMSE - trueCDF).^2) mean(abs(cdfMSE - trueCDF))];
        
        resultsMSE_r(m, 3:4) = [sqrt(mean((thetaMSE - thetaTrue).^2)) mean(abs(thetaMSE - thetaTrue))];
    
        % number of positive, true positive and sign consistent weights 
        
        thetaMSE = thetaMSE .* (thetaMSE > 1e-3);
        
        resultsMSE_r(m, 5:7) = [sum(thetaMSE > 0) (sum((thetaMSE > 0) .* (thetaTrue > 0)) / S)*100 (sum(sign(thetaMSE) == sign(thetaTrue)) / R)*100];
        
        % save tuning parameter 
        
        resultsMSE_r(m, 8) = muMSE;
        
        % LL
        
        [thetaLL, ~] = estimator(choiceProbsR, dependentData, muLL, 1e-7);

        cdfLL = estcdf(thetaLL, gridR, gridE);
        
        % calculate RMISE IAE, L1 and L2 norm
        
        resultsLL_r(m, 1:2) = [mean((cdfLL - trueCDF).^2) mean(abs(cdfLL - trueCDF))];
        
        resultsLL_r(m, 3:4) = [sqrt(mean((thetaLL - thetaTrue).^2)) mean(abs(thetaLL - thetaTrue))];
    
        % number of positive, true positive and sign consistent weights 
        
        thetaLL = thetaLL .* (thetaLL > 1e-3);
        
        resultsLL_r(m, 5:7) = [sum(thetaLL > 0) (sum((thetaLL > 0) .* (thetaTrue > 0)) / S)*100 (sum(sign(thetaLL) == sign(thetaTrue)) / R)*100];
        
        % save tuning parameter 
        
        resultsLL_r(m, 8) = muLL;
        
        
    end
       
    % calculate averages over monte carlo replicates
    
    resultsFKRB(r, 4:10) = [sqrt(mean(resultsFKRB_r(:, 1))) mean(resultsFKRB_r(:, 2:7), 1)];
    
    resultsMSE(r, 4:11) = [sqrt(mean(resultsMSE_r(:, 1))) mean(resultsMSE_r(:, 2:8), 1)];
    
    resultsOneSe(r, 4:11) = [sqrt(mean(resultsOneSe_r(:, 1))) mean(resultsOneSe_r(:, 2:8), 1)];
    
    resultsLL(r, 4:11) = [sqrt(mean(resultsLL_r(:, 1))) mean(resultsLL_r(:, 2:8), 1)];
    
  
    % show progress
    
    fprintf('Monte Carlo study for N = %d and R = %d finished \n', n, R);
        
         
end

    
   
% save results as text file

cd '/home/osterhau/Nonparametric_Fox/Code_Matlab/'

headerFKRB = {'N', 'R', 'S', 'RMISE', 'IAE', 'L2', 'L1', 'Pos', 'TruePos', 'Sign'};
resultsFKRB = array2table(resultsFKRB, 'VariableNames', headerFKRB);
writetable(resultsFKRB, 'resultsFKRB.txt', 'delimiter', ';');

headerENet = {'N', 'R', 'S', 'RMISE', 'IAE', 'L2', 'L1', 'Pos', 'TruePos', 'Sign', 'mu'};
resultsMSE = array2table(resultsMSE, 'VariableNames', headerENet);
writetable(resultsMSE, 'resultsMSE.txt', 'delimiter', ';');

resultsOneSe = array2table(resultsOneSe, 'VariableNames', headerENet);
writetable(resultsOneSe, 'resultsOneSe.txt', 'delimiter', ';');

resultsLL = array2table(resultsLL, 'VariableNames', headerENet);
writetable(resultsLL, 'resultsLL.txt', 'delimiter', ';');






