


function [muMSE, muOneSe, muLL] = crossValidation(choiceProbsR, dependentData, mus, nFolds, j)


    % specify total number of observations, number of observation units and number of tuning parameter
    
    n = size(choiceProbsR, 1);     nObs = n / j;       nMus = length(mus);
    
   
    % randomly assign observation units to folds
    
    ids = [transpose(1:nObs) reshape(1:n, [j nObs])'];
    
    idx = reshape(ids(randsample(nObs, nObs, false), 2:(j + 1))', [nObs * j, 1]);
    
    clear ids
    
    % shuffle data 
    
    choiceProbsR = choiceProbsR(idx, :);
    
    dependentData = dependentData(idx);
    
    % create equally sized folds
    
    nMissing = nObs - floor(nObs / nFolds) * nFolds;
    
    if(nMissing == 0) 
        
        folds = repelem(1:nFolds, j * nObs / nFolds)';
        
    else 
        
        folds = [repelem(1:nFolds, j * floor(nObs / nFolds)) repelem(1:nMissing, 1)]';
        
    end
    
  
    % empty matrix to save cross validation error
  
    Metrics = zeros(nMus, 2);
  
    cvStandardError = zeros(nMus, 1);
 
    % calculate cross validation error for every mu
  
    for i = 1:nMus
        
        % empty matrix to save cv error for every fold 
    
        MetricsFold = zeros(nFolds, size(Metrics, 2));
    
        % loop over every fold
        
        for f = 1:nFolds
     
            % select training data
      
            idx = find(folds == f);
     
            choiceProbsR_f = choiceProbsR(folds ~= f, :);
      
            dependentData_f = dependentData(folds ~= f);
      
            % estimate coefficients
    
            [theta, ~] = estimator(choiceProbsR_f, dependentData_f, mus(i), 1e-7);
        
            % calculate predicted out-of-sample choice probabilities
  
            dependentDataHat = choiceProbsR(idx, :) * theta;
  
            % calculate out-of-sample MSE
      
            MetricsFold(f, 1) = mean((dependentData(idx) - dependentDataHat).^2); 
            
            % calculate log-likelihood
      
            MetricsFold(f, 2) = log(dependentDataHat)' * dependentData(idx);
            
        end
        
        % calculate average over folds
    
        Metrics(i, :) = mean(MetricsFold, 1);
    
        % calculate standard error for OneSe rule
    
        cvStandardError(i) = std(MetricsFold(:, 1)) / sqrt(nFolds);
               
    end
    
    % select mu value according to different criteria
    
    indMin = find(Metrics(:, 1) == min(Metrics(:, 1)), 1, 'first');
  
    muMSE = mus(indMin);
  
    muOneSe = max(mus(Metrics(:, 1) <= Metrics(indMin, 1) + cvStandardError(indMin)));

    muLL = max(mus(Metrics(:, 2) == max(Metrics(:, 2))));
  
end


