    

function [thetas, exitflag] = estimator(choiceProbsR, dependentData, mu, eps)
    
    
    % specify number of grid points
    
    R = size(choiceProbsR, 2);
   
    
    % specify constraints
    
    lb = zeros(R, 1);           ub = ones(R, 1);
        
    beq = 1;                    Aeq = ones(1, R);

    % transform choiceProbsR and dependentData for elastic net
            
    X = [choiceProbsR; diag(ones(R, 1)) .* sqrt(mu)];
    
    y = [dependentData; zeros(R, 1)];
        
   % define options for lsqlin function
   
   options = optimset('algorithm', 'interior-point', 'MaxIter', 10000, 'TolCon', 1e-10, 'TolFun', 1e-10, 'Display', 'off');
        
   % estimate weights with lsqlin function
   
   [thetas, ~, ~, exitflag, ~, ~] =  lsqlin(X, y, Aeq, beq, [], [], lb, ub, [], options);

   % check if estimator converged
        
   if exitflag ~= 1
       
       disp('lsqlin did not converge successfully');
       
       thetas = zeros(R, 1);

   end

   % set all weights < eps to exactly zero and r
        
   thetas = thetas .* (thetas > eps);
    
end

        