


crossValidation = function(choiceProbsR, dependentData, mus, nFolds, j) {
  
  
  
  # specify number of tuning parameter, number of grid points and number of observation units
  
  nMus = length(mus);     R = ncol(choiceProbsR);     nObs = nrow(choiceProbsR)/j
  

  # shuffle data 

  ids = cbind(1:nObs, matrix(1:nrow(choiceProbsR), nrow = nObs, ncol = j, byrow = T))
    
  idx = c(t(ids[sample(nObs), 2:(1 + j)]))  

  rm(ids)
  
  choiceProbsR  = choiceProbsR[idx, ]
  
  dependentData = dependentData[idx]
  

  # create k equally sized folds

  folds = rep(cut(seq(1:nObs), breaks = nFolds, labels = F), each = j)

  
  # set up empty vector to save cross validation metrics 
  
  Metrics = data.frame(MSE = rep(0, nMus), LL = 0, MAD = 0)
  
  cvStandardError = numeric(length = nMus)
 
  
  # loop over tuning parameter mu

  for(i in 1:nMus) {

    # create emtpy vector to save cv error for every fold 
    
    MetricsFold = matrix(0, nrow = nFolds, ncol = ncol(Metrics))
    
    # loop over folds
    
    for(f in 1:nFolds) {
    
      # select training data
  
      idx = which(folds == f, arr.ind = TRUE)

      choiceProbsR_f = choiceProbsR[-idx, ]

      dependentData_f = dependentData[-idx]
      
      
      # estimate coefficients
      
      theta = estimator(choiceProbsR_f, dependentData_f, mus[i], j, eps = 1e-05)$theta
        
      
      # calculate predicted out-of-sample choice probabilities 
  
      dependentDataHat = choiceProbsR[idx, ] %*% theta
  
      # calculate test error
  
      MetricsFold[f, 1] = mean((dependentData[idx] - dependentDataHat)^2) 

      # calculate log likelihood
      
      MetricsFold[f, 2] = t(log(dependentDataHat)) %*% dependentData[idx]
      
      # predict binary variable 
      
      dependentDataHatMatrix = matrix(dependentDataHat, nrow = length(idx)/j, ncol = j, byrow = T)
      
      dependentDataHatMatrixCat = +(dependentDataHatMatrix == dependentDataHatMatrix[cbind(1:nrow(dependentDataHatMatrix), 
                                                                                           
                                                                                           max.col(dependentDataHatMatrix))])
      dependentDataHatCat = c(t(dependentDataHatMatrixCat))
      
    
      # calculate out-of-sample prediction error
      
      MetricsFold[f, 3] = sum(sign(dependentData[idx] != sign(dependentDataHatCat))) / length(idx)
      
      
    }
    
    
    # calculate average metrics over folds
  
    Metrics[i, ] = colMeans(MetricsFold)
  
    # calculate standard error for OneSe rule
    
    cvStandardError[i] = sd(MetricsFold[, 1]) / sqrt(nFolds)
  
  }
  
  
  # select mu values according to different criteria
  
  # min MSE
  
  indMin = max(which(Metrics[, 1] == min(Metrics[, 1], na.rm = T)))
  
  muMSE = mus[indMin]

  # OneSe
  
  muOnseSe = max(mus[which(Metrics[, 1] <= Metrics[indMin, 1] + cvStandardError[indMin])], na.rm = T)

  # max LL
  
  muLL = max(mus[which(Metrics[, 2] == max(Metrics[, 2]))])

  # min prediction error
  
  muCatError = max(mus[which(Metrics[, 3] == min(Metrics[, 3]))])
  
  
  # return mu values
  
  return(list(muMSE = muMSE, muOneSe = muOnseSe, muLL = muLL, muCatError = muCatError))

}


  
  
  