

gendata = function(n, j, betasTrue) {
  
  # specify dimension of random coefficients beta
  
  k = ncol(betasTrue)
  
  
  # define iota vector for matrix multiplication                  
  
  iota = matrix(1, nrow = j, ncol = 1)

  
  # draw betas from true distribution

  betasTrue = betasTrue[sample(nrow(betasTrue), n, replace = FALSE), ]
  
  
  # draw product characteristics from uniform distributions
  
  xdata = cbind(runif(n * j, min = 0, max = 5), runif(n * j, min = -3, max = 1))
  
  # calculate linear index
  
  linearIndex = betasTrue[rep(1:nrow(betasTrue), each = j), ] * xdata
  
  linearIndex = linearIndex %*% matrix(1, nrow = k, ncol = 1)
  

  # calculate logit kernels
  
  expLinearIndex = matrix(exp(linearIndex), nrow = n, ncol = j, byrow = T)
  
  expLinearIndexSum = 1 + (expLinearIndex %*% iota)
  
  choiceProbs = expLinearIndex / c(expLinearIndexSum)
  
  
  # add share of outside option
  
  outsideShare = 1 - (choiceProbs %*% iota)
  
  choiceProbsWithOutside = cbind(choiceProbs, outsideShare)
  
  
  # generate choices

  choiceData = integer(length = n)

  for(i in 1:n) {
    
    q = cumsum(choiceProbsWithOutside[i, ]) / sum(choiceProbsWithOutside[i, ])
    
    rs = runif(1, min = 0, max = 1)
    
    choiceData[i] = sum(rs > q) + 1
    
  }
  
  
  # create binary dependent variable for linear probabolity model
  
  dependentDataMat = matrix(0, nrow = n, ncol = j + 1)
  
  dependentDataMat[cbind(1:n, choiceData)] = 1
  
  
  # rearrange data for regression
  
  dependentData = c(t(dependentDataMat[, 1:j]))
  
  
  # return data
  
  return(list(betasTrue = betasTrue, xdata = xdata, dependentData = dependentData))
  
  
  
  
}
  
  
  
  
  
  