
set.seed(12345)

library(MASS);    library(limSolve);    library(mvtnorm);   library(glmnet);    library(Matrix)


# load functions

setwd("/Functions")

source("halton_draws.R");     source("gendata.R");      source("cdfEst.R");             source("cdfTrue.R")

source("gmdist.R");           source("estimator.R");    source("crossValidation.R")

setwd("/Monte_Carlos/MC_Continuous/")

source("MC_Continuous_function.R")


# specify normals of true distribution

means = matrix(c(-2.2, -2.2, 1.3, 1.3), nrow = 2, ncol = 2, byrow = TRUE)

sigmas = array(c(0.8, 0.15, 0.15, 0.8, 0.8, 0.15, 0.15, 0.8), dim = c(2, 2, 2))

probs = c(0.5, 0.5)


# draw betas from true distribution for data generation

betasTrue = gmdist(500000, means, sigmas, probs)



# specify parameter of Monte Carlo experiments

Nvec =  c(1000,10000);    Rvec = c(25, 50, 100, 250)


# define parameters to generate data

j = 4;    k = 2;   E = sqrt(10000);  nFolds = 10


# set up a grid for evaluations

gridE = as.matrix(expand.grid(seq(-4.5, 3.5, length.out = E), seq(-4.5, 3.5, length.out = E)))

# calculate true cdf

trueCDF = numeric(length = nrow(gridE))

for(i in 1:E^2) {
  
  trueCDF[i] = probs[1] * pmvnorm(lower = -Inf, upper = gridE[i, ], mean = means[1, ], sigma = sigmas[, , 1]) + 
    
    probs[2] * pmvnorm(lower = -Inf, upper = gridE[i, ], mean = means[2, ], sigma = sigmas[, , 2])
  
}


# get arguments from batch job

args = (commandArgs(TRUE))

mcr = as.integer(args[1]) + 1


# sepcify path to save function output 

path = ''


# conduct Monte Carlo experiments (To replicate the Monte Carlo experiments 200 times, you have to call the function MC_Continuous() 200 times.
# We ran the Monte Carlo experiment on a HPC and sent this file, MC_Continuous_Parallel.R, 200 times to the cluster.)

MC_Continuous(path, mcr, Nvec, Rvec)


