





MC_Discrete = function(path, mcr, Nvec, Rvec){
  
  
  # set seed
  
  set.seed(12345 + mcr)
  
  
  # create empty dataframes to save results
  
  params = data.frame(N = rep(Nvec, each = length(Rvec)), R = rep(Rvec, times = length(Nvec)))
  
  outputFKRB_r = data.frame(params, S = 0, RMISE = 0, IAE = 0, Pos = 0, TruePos = 0, Sign = 0, L1 = 0, L2 = 0, rho = 0)
  
  outputMSE_r = data.frame(params, S = 0, RMISE = 0, IAE = 0, Pos = 0, TruePos = 0, Sign = 0, L1 = 0, L2 = 0, Mu = 0)
  
  outputLL_r = outputOneSe_r = outputCatError_r = outputMSE_r
  
  CorMatrix_r = data.frame(Perc25 = rep(0, nrow(params)), Median = 0, Mean = 0, Perc75 = 0, Max = 0, nGreater95  = 0)
  
  
  # loop over different specifications
  
  for(r in 1:nrow(params)) {
    
    
    # specify parameter of Monte Carlo experiment
    
    n = params[r, 'N'];   R = params[r, 'R']
    
    
    # print starting message
    
    t1 = Sys.time()
    
    cat(paste("Monte Carlo Experiment for for N = ", n,", R = ", R," started at ", t1, "\n ", sep = ""))
    
    cat('\n')
    
    
    # specify uniform grid for estimation
    
    gridR = expand.grid(seq(-4.5, 3.5, length.out = sqrt(R)), seq(-4.5, 3.5, length.out = sqrt(R)))
    
    
    # select grid points for data generation 
    
    gridIdx = which(gridR[, 1] <= -0.5 & gridR[, 2] <= -0.5 | gridR[, 1] >= -0.5 & gridR[, 2] >= -0.5)
    
    S = length(gridIdx)
    
    
    # calculate weights and true distribution 
    
    thetaStar = numeric(length = R)
    
    thetaStar[gridIdx] = 1/S;		thetaNorm = thetaStar
    
    trueCDF = cdfEst(gridE, gridR, thetaStar)
    
    
    # save number of true points
    
    outputFKRB_r[r, 'S'] = outputMSE_r[r, 'S'] = outputOneSe_r[r, 'S'] = outputLL_r[r, 'S'] = outputCatError_r[r, 'S'] = S
    
    
    # generate data 
    
    betasTrue = as.matrix(gridR[sample(gridIdx, n, replace = TRUE, prob = thetaStar[gridIdx]), ])
    
    data = gendata(n, j, betasTrue)
    
    xdata = data$xdata
    
    dependentData = data$dependentData
    
    rm(data)
    
    
    # calculate choice probabilities at every grid point
    
    linearIndex = xdata %*% t(gridR)
    
    expLinearIndex = exp(linearIndex)
    
    expLinearIndexSum = 1 + (matrix(rep(diag(1, n), each = j), nrow = n, ncol = j * n, byrow = TRUE) %*% expLinearIndex)
    
    choiceProbsR = expLinearIndex / expLinearIndexSum[rep(1:n, each = j), ]
    
    rm(expLinearIndexSum, expLinearIndex, linearIndex)
    
    
    # calculate summary statistics of correlation among choice probabilities
    
    cMatO = cor(choiceProbsR, method = "spearman")
    
    cMat = abs(cMatO[upper.tri(cMatO, diag = FALSE)] )
    
    cMat = cMat[is.na(cMat) == FALSE]
    
    CorMatrix_r[r, ] = c(summary(cMat)[-1], sum(cMat > 0.95) / length(cMat))
    
    rm(cMat, cMatO)
    
    
    
    # estimate weights with FKRB estimator and calculate joint CDF
    
    thetaFKRB = estimator(choiceProbsR, dependentData, 0, j, eps = 1e-07)$theta
    
    EstCDF = cdfEst(gridE, gridR, thetaFKRB)
    
    # calculate RMISE, IAE, and L1 and L2 norm 
    
    outputFKRB_r[r, c('RMISE', 'IAE')] = c(mean((EstCDF - trueCDF)^2), mean(abs(EstCDF - trueCDF)))
    
    outputFKRB_r[r, c('L1', 'L2')] = c(mean(abs(thetaFKRB - thetaNorm)), sqrt(sum((thetaFKRB - thetaNorm)^2)) / R)
    
    # calculate number of positive estimated weights and share of true positive and sign consistent weights
    
    thetaFKRB = thetaFKRB * (thetaFKRB > 1e-3)
    
    outputFKRB_r[r, c('Pos', 'TruePos', 'Sign')] = c(sum(thetaFKRB > 0), (sum((thetaFKRB > 0) * (thetaStar > 0)) / S) * 100, (sum(sign(thetaFKRB) == sign(thetaStar)) / R) * 100)
    
    
    
    # generate sequence of tuning parameter
    
    mus = c(0, glmnet(choiceProbsR, dependentData, alpha = 0, intercept = FALSE, lower.limits = rep(0, R), standardize = FALSE)$lambda)
    
    # find optimal tuning parameter with k-fold cross validation
    
    CvMus = crossValidation(choiceProbsR, dependentData, mus, nFolds, j)
    
    
    # estimate weights with generalized estimator using different tuning parameter
    
    # min MSE
    
    thetaMSE = estimator(choiceProbsR, dependentData, CvMus$muMSE, j, eps = 1e-07)$theta
    
    EstCDFMSE = cdfEst(gridE, gridR, thetaMSE)
    
    # calculate RMISE, IAE, and L1 and L2 norm 
    
    outputMSE_r[r, c('RMISE', 'IAE')] = c(mean((EstCDFMSE - trueCDF)^2), mean(abs(EstCDFMSE - trueCDF)))
    
    outputMSE_r[r, c('L1', 'L2')] = c(mean(abs(thetaMSE - thetaNorm)), sqrt(sum((thetaMSE - thetaNorm)^2)) / R)
    
    # calculate number of positive estimated weights and share of true positive and sign consistent weights
    
    thetaMSE = thetaMSE * (thetaMSE > 1e-3)
    
    outputMSE_r[r, c('Pos', 'TruePos', 'Sign')] = c(sum(thetaMSE > 0), (sum((thetaMSE > 0) * (thetaStar > 0)) / S) * 100, (sum(sign(thetaMSE) == sign(thetaStar)) / R) * 100)
    
    
    
    # OneSeRule
    
    thetaOneSe = estimator(choiceProbsR, dependentData, CvMus$muOneSe, j, eps = 1e-07)$theta
    
    EstCDFOneSe = cdfEst(gridE, gridR, thetaOneSe)
    
    # calculate RMISE, IAE, and L1 and L2 norm 
    
    outputOneSe_r[r, c('RMISE', 'IAE')] = c(mean((EstCDFOneSe - trueCDF)^2), mean(abs(EstCDFOneSe - trueCDF)))
    
    outputOneSe_r[r, c('L1', 'L2')] = c(mean(abs(thetaOneSe - thetaNorm)), sqrt(sum((thetaOneSe - thetaNorm)^2)) / R)
    
    # calculate number of positive estimated weights and share of true positive and sign consistent weights
    
    thetaOneSe = thetaOneSe * (thetaOneSe > 1e-3)
    
    outputOneSe_r[r, c('Pos', 'TruePos', 'Sign')] = c(sum(thetaOneSe > 0), (sum((thetaOneSe > 0) * (thetaStar > 0)) / S) * 100, (sum(sign(thetaOneSe) == sign(thetaStar)) / R) * 100)
    
    
    # max LL
    
    thetaLL = estimator(choiceProbsR, dependentData, CvMus$muLL, j, eps = 1e-07)$theta
    
    EstCDFLL = cdfEst(gridE, gridR, thetaLL)
    
    # calculate RMISE, IAE, and L1 and L2 norm 
    
    outputLL_r[r, c('RMISE', 'IAE')] = c(mean((EstCDFLL - trueCDF)^2), mean(abs(EstCDFLL - trueCDF)))
    
    outputLL_r[r, c('L1', 'L2')] = c(mean(abs(thetaLL - thetaNorm)), sqrt(sum((thetaLL - thetaNorm)^2)) / R)
    
    # calculate number of positive estimated weights and share of true positive and sign consistent weights
    
    thetaLL = thetaLL * (thetaLL > 1e-3)
    
    outputLL_r[r, c('Pos', 'TruePos', 'Sign')] = c(sum(thetaLL > 0), (sum((thetaLL > 0) * (thetaStar > 0)) / S) * 100, (sum(sign(thetaLL) == sign(thetaStar)) / R) * 100)
    
    
    # min prediction error
    
    thetaCatError = estimator(choiceProbsR, dependentData, CvMus$muCatError, j, eps = 1e-07)$theta
    
    EstCDFCatError = cdfEst(gridE, gridR, thetaCatError)
    
    # calculate RMISE, IAE, and L1 and L2 norm 
    
    outputCatError_r[r, c('RMISE', 'IAE')] = c(mean((EstCDFCatError - trueCDF)^2), mean(abs(EstCDFCatError - trueCDF)))
    
    outputCatError_r[r, c('L1', 'L2')] = c(mean(abs(thetaCatError - thetaNorm)), sqrt(sum((thetaCatError - thetaNorm)^2)) / R)
    
    # calculate number of positive estimated weights and share of true positive and sign consistent weights
    
    thetaCatError = thetaCatError * (thetaCatError > 1e-3)
    
    outputCatError_r[r, c('Pos', 'TruePos', 'Sign')] = c(sum(thetaCatError > 0), (sum((thetaCatError > 0) * (thetaStar > 0)) / S) * 100, (sum(sign(thetaCatError) == sign(thetaStar)) / R) * 100)
    
    
    # save tuning parameter
    
    outputMSE_r[r, 'Mu'] = CvMus$muMSE
    
    outputOneSe_r[r, 'Mu'] = CvMus$muOneSe
    
    outputLL_r[r, 'Mu'] = CvMus$muLL
    
    outputCatError_r[r, 'Mu'] = CvMus$muCatError 
    
    # save correlation
    
    outputFKRB_r[r, 'rho'] = CorMatrix_r[r, "Perc75"]
    
    
    
    # time and print progress
    
    t2 = Sys.time()
    
    h = round(difftime(t2, t1, units = "hours")[[1]], 2)
    
    cat(paste("Calculation for N = ", n, " and R = ", R, " finished. Run Time: ", h, " \n ", sep = ""))
    
    cat(" \n")
    
    cat("--------------------------------------------------------------------------- \n")
    
    cat(" \n") 
    
    
    
  }
  
  
  # save in list 
  
  out_r = list(outputFKRB_r = outputFKRB_r, outputMSE_r = outputMSE_r, outputOneSe_r = outputOneSe_r, outputLL_r = outputLL_r, outputCatError_r = outputCatError_r,
                CorMatrix_r = CorMatrix_r)
  
  # save out_r
  
  setwd(path)
  
  save(out_r, file = paste('out_mcr_', mcr, '.RData', sep = ''))
  
  # print warnings
  
  cat('\n')
  cat('Warnings: \n')
  cat('------------ \n')
  cat('\n')
  warnings()
  
  
  
}

